from _typeshed import Incomplete
from storage_connection_parameters import StorageConnectionParameters as StorageConnectionParameters

class S3Client:
    storage_connection_parameters: Incomplete
    def __init__(self, storage_connection_parameters: StorageConnectionParameters) -> None: ...
    def upload_zipped_jsoned_string(self, data: str, file_name: str, content_type: str = 'application/octet-stream') -> None: ...
    def upload_existing_file(self, file_path: str) -> None: ...
    def shutdown(self) -> None: ...
