from dataclasses import dataclass

@dataclass(slots=True)
class StorageConnectionParameters:
    azure_blob_parameters_with_key: str | None = ...
    azure_container_name: str | None = ...
    backblaze_access_key_id: str | None = ...
    backblaze_secret_access_key: str | None = ...
    backblaze_endpoint_url: str | None = ...
    backblaze_bucket_name: str | None = ...
    def __post_init__(self) -> None: ...
